(function (global) {

    var TWO_PI = Math.PI * 2,
        circleStroke = '#e5b2b2',
        circleFill = 'rgba(229, 178, 178, 0.5)',
        rectStroke = '#7fe57f',
        rectFill = 'rgba(127, 229, 127, 0.5)';

    /**
     * Helper class for Box2D body paint
     * @param {HTMLCanvasElement} canvas Canvas element on which debug content will be drawn
     */
    function Painter(canvas) {
        this.canvas = canvas;
        this.ctx = this.canvas.getContext('2d');
        this.width = this.canvas.width;
        this.height = this.canvas.height;
        this.scale = 10;
    }

    /**
     * Draw box2d bodies on canvas
     * @param {Objec} world Box2D world
     */
    Painter.prototype.paint = function (world) {
        var body = world.GetBodyList();

        this.ctx.clearRect(0, 0, this.width, this.height);

        while (body) {
            if (body.m_fixtureList) {
                if (body.m_fixtureList.m_shape.constructor.name === 'b2PolygonShape') {
                    this.paintRectangle(body);
                } else if (body.m_fixtureList.m_shape.constructor.name === 'b2CircleShape') {
                    this.paintCircle(body);
                }
            }

            body = body.GetNext();
        }
    };

    /**
     * Draw a circle
     * @param {Object} body Box2D body object
     */
    Painter.prototype.paintCircle = function (body) {
        var x = body.m_xf.position.x * this.scale,
            y = body.m_xf.position.y * this.scale,
            r = body.m_fixtureList.m_shape.m_radius * this.scale;
        
        this.ctx.beginPath();
        this.ctx.arc(x, y, r, 0, TWO_PI, false);
        this.ctx.fillStyle = circleFill;
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = circleStroke;
        this.ctx.stroke();
    };

    /**
     * Draw a rectangle
     * @param {Object} body Box2D body object
     */
    Painter.prototype.paintRectangle = function (body) {
        var v = body.m_fixtureList.m_shape.m_vertices,
            x = (body.m_xf.position.x + v[0].x) * this.scale,
            y = (body.m_xf.position.y + v[0].y) * this.scale,
            w = (v[2].x - v[0].x) * this.scale,
            h = (v[2].y - v[0].y) * this.scale;

        this.ctx.beginPath();
        this.ctx.rect(x, y, w, h);
        this.ctx.fillStyle = rectFill;
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = rectStroke;
        this.ctx.stroke();
    };

    global.Painter = Painter;

}(this));