(function (global) {

    var TWO_PI = Math.PI * 2,
        circleStroke = '#e5b2b2',
        circleFill = 'rgba(229, 178, 178, 0.5)',
        rectStroke = '#7fe57f',
        rectFill = 'rgba(127, 229, 127, 0.5)';

    /**
     * Helper class for Chipmunk shape paint
     * @param {HTMLCanvasElement} canvas Canvas element on which debug content will be drawn
     */
    function Painter(canvas) {
        this.canvas = canvas;
        this.ctx = this.canvas.getContext('2d');
        this.width = this.canvas.width;
        this.height = this.canvas.height;
        this.scale = 10;
    }

    /**
     * Draw Chipmunk shapes on canvas
     * @param {Objec} space Chipmunk space to draw
     */
    Painter.prototype.paint = function (space) {
        var that = this;

        this.ctx.clearRect(0, 0, this.width, this.height);

        space.eachShape(function (shape) {
            if (shape.type === 'circle') {
                that.paintCircle.call(that, shape);
            } else if (shape.type === 'segment') {
                that.paintRectangle.call(that, shape);
            }
        });
    };

    /**
     * Draw a circle
     * @param {Object} shape Chipmunk shape object
     */
    Painter.prototype.paintCircle = function (shape) {
        var x = (shape.bb_l + shape.r) * this.scale,
            y = (shape.bb_b + shape.r) * this.scale,
            r = shape.r * this.scale;
        
        this.ctx.beginPath();
        this.ctx.arc(x, y, r, 0, TWO_PI, false);
        this.ctx.fillStyle = circleFill;
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = circleStroke;
        this.ctx.stroke();
    };

    /**
     * Draw a rectangle
     * @param {Object} shape Chipmunk shape object
     */
    Painter.prototype.paintRectangle = function (shape) {
        var x = shape.bb_l * this.scale,
            y = shape.bb_b * this.scale,
            w = (shape.bb_r - shape.bb_l) * this.scale,
            h = (shape.bb_t - shape.bb_b) * this.scale;

        this.ctx.beginPath();
        this.ctx.rect(x, y, w, h);
        this.ctx.fillStyle = rectFill;
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = rectStroke;
        this.ctx.stroke();
    };

    global.Painter = Painter;

}(this));