(function (global) {

    var TWO_PI = Math.PI * 2,
        circleStroke = '#e5b2b2',
        circleFill = 'rgba(229, 178, 178, 0.5)',
        rectStroke = '#7fe57f',
        rectFill = 'rgba(127, 229, 127, 0.5)';

    /**
     * Helper class for Turbulenz body paint
     * @param {HTMLCanvasElement} canvas Canvas element on which debug content will be drawn
     */
    function Painter(canvas) {
        this.canvas = canvas;
        this.ctx = this.canvas.getContext('2d');
        this.width = this.canvas.width;
        this.height = this.canvas.height;
        this.scale = 10;
    }

    /**
     * Draw rigid bodies on canvas
     * @param {Array} bodies Array of rigid bodies
     */
    Painter.prototype.paint = function (bodies) {
        var len = bodies.length,
            body,
            i;

        this.ctx.clearRect(0, 0, this.width, this.height);

        for (i = 0; i < len; i++) {
            body = bodies[i];
            if (body.shapes[0].type === 'POLYGON') {
                this.paintRectangle(body);
            } else if (body.shapes[0].type === 'CIRCLE') {
                this.paintCircle(body);
            }
        }
    };

    /**
     * Draw a circle
     * @param {Object} body RigidBody object
     */
    Painter.prototype.paintCircle = function (body) {
        var data = body.shapes[0]._data,
            x = (data[0] + data[4]) * this.scale,
            y = (data[1] + data[4]) * this.scale,
            r = data[4] * this.scale; // 0 - px, 1 - py, r - 4
        
        this.ctx.beginPath();
        this.ctx.arc(x, y, r, 0, TWO_PI, false);
        this.ctx.fillStyle = circleFill;
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = circleStroke;
        this.ctx.stroke();
    };

    /**
     * Draw a rectangle
     * @param {Object} body RigidBody object
     */
    Painter.prototype.paintRectangle = function (body) {
        var data = body.shapes[0]._data,
            x = (data[0]) * this.scale,
            y = (data[1]) * this.scale,
            w = (data[2] - data[0]) * this.scale,
            h = (data[3] - data[1]) * this.scale;

        this.ctx.beginPath();
        this.ctx.rect(x, y, w, h);
        this.ctx.fillStyle = rectFill;
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = rectStroke;
        this.ctx.stroke();
    };

    global.Painter = Painter;

}(this));